
abstract class Validatable{
  DataResult validate();
}


class DataResult<T>{
  DataResult({required this.success, required this.message, this.data, required this.code});
  final bool success;
  final String message;
  final String code;
  final T? data;
}

abstract class Sortable{
  int sortWeight();
}

/**
 * Base response class
 */
class BaseResponse<T>{
  BaseResponse({
    required this.success,
    required this.message,
    this.data,
    required this.code,
    this.metadata
  });

  final bool success; // Indicating if the response is successful
  final String message; // Message of the response
  final String code; // Unique code of the response
  final T? data; // Optional data of the response
  final Map<String, dynamic>? metadata; // Optional metadata of the response
}

/**
 * Base response class
 */
class BaseListResponse<T> extends BaseResponse<List<T>>{
  BaseListResponse({
    required bool success,
    required String message,
    required String code,
    List<T>? data,
    this.page,
    Map<String, dynamic>? metadata
  }): super(
    success: success,
    message: message,
    code: code,
    data: data,
    metadata: metadata
  );

  final BasePaginationResponse? page;
}

/**
 * Base response for pagination
 */
class BasePaginationResponse{
  BasePaginationResponse({
    this.start,
    this.end,
    this.limit,
    this.pageCount,
    this.total,
    this.page
  });

  final int? start; // Start position of the records
  final int? end; // End position of the records
  final int? page; // Current page of the records
  final int? limit; // Limit of the records for each page
  final int? pageCount; // Number of page count based on the limit
  final int? total; // Total number of records
}